@@ -1,6 +1,8 @@
 STRUCTURED_TYPE = 'structured'
 ML_MODEL_FILE_NAME_KEY = "ML_MODEL_FILE_NAME"
 ML_MODEL_FILE_NAME_VALUE = "model"
+SYS_DEPLOYMENT_FILE_NAME_KEY = "SYS_DEPLOYMENT_FILE_NAME"
+SYS_DEPLOYMENT_FILE_NAME_VALUE = "deployment"
 
 STATUS_INITIALISED = 'initialised'
 SUCCESS_STATUSES = ['completed', 'available', 'success']
